@@ -10,14 +10,21 @@
  * Read the configuration and load the appropriate drivers.
  */
 #include "config.h"
-#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <dlfcn.h>
+
+#if !defined(HAVE_STRSEP)
+#include "ts_strsep.h"
+#endif
 
 #include "tslib-private.h"
 
-/* Maximum line size is BUF_SIZE - 2 
+/* Maximum line size is BUF_SIZE - 2
  * -1 for fgets and -1 to test end of line
  */
 #define BUF_SIZE 512
@@ -25,11 +32,14 @@
 /* Discard any empty tokens
  * Note: strsep modifies p (see man strsep)
  */
-void discard_null_tokens(char **p, char **tokPtr)
+static void discard_null_tokens(char **p, char **tokPtr)
 {
-	while (*p != NULL && **tokPtr == '\0')
-	{
+	while (*p != NULL && **tokPtr == '\0') {
+	#if !defined HAVE_STRSEP
+		*tokPtr = ts_strsep(p, "\t");
+	#else
 		*tokPtr = strsep(p, " \t");
+	#endif
 	}
 }
 
@@ -39,15 +49,25 @@ int ts_config(struct tsdev *ts)
 	FILE *f;
 	int line = 0;
 	int ret = 0;
+	short strdup_allocated = 0;
 
 	char *conffile;
 
-	if( (conffile = getenv("TSLIB_CONFFILE")) == NULL) {
-		conffile = strdup (TS_CONF);
+	if ((conffile = getenv("TSLIB_CONFFILE")) == NULL) {
+		conffile = strdup(TS_CONF);
+		if (conffile) {
+			strdup_allocated = 1;
+		} else {
+			perror("Couldn't find tslib config file");
+			return -1;
+		}
 	}
 
 	f = fopen(conffile, "r");
 	if (!f) {
+		if (strdup_allocated)
+			free(conffile);
+
 		perror("Couldnt open tslib config file");
 		return -1;
 	}
@@ -62,9 +82,8 @@ int ts_config(struct tsdev *ts)
 
 		/* Chomp */
 		e = strchr(p, '\n');
-		if (e) {
+		if (e)
 			*e = '\0';
-		}
 
 		/* Did we read a whole line? */
 		if (buf[BUF_SIZE - 2] != '\0') {
@@ -72,27 +91,39 @@ int ts_config(struct tsdev *ts)
 			break;
 		}
 
+	#if !defined HAVE_STRSEP
+		tok = ts_strsep(&p, " \t");
+	#else
 		tok = strsep(&p, " \t");
+	#endif
 		discard_null_tokens(&p, &tok);
 
 		/* Ignore comments or blank lines.
 		 * Note: strsep modifies p (see man strsep)
 		 */
-		if (p==NULL || *tok == '#')
+		if (p == NULL || *tok == '#')
 			continue;
 
 		/* Search for the option. */
 		if (strcasecmp(tok, "module") == 0) {
+		#if !defined HAVE_STRSEP
+			module_name = ts_strsep(&p, " \t");
+		#else
 			module_name = strsep(&p, " \t");
+		#endif
 			discard_null_tokens(&p, &module_name);
 			ret = ts_load_module(ts, module_name, p);
-		}
-		else if (strcasecmp(tok, "module_raw") == 0) {
+		} else if (strcasecmp(tok, "module_raw") == 0) {
+		#if !defined HAVE_STRSEP
+			module_name = ts_strsep(&p, " \t");
+		#else
 			module_name = strsep(&p, " \t");
+		#endif
 			discard_null_tokens(&p, &module_name);
 			ret = ts_load_module_raw(ts, module_name, p);
 		} else {
-			ts_error("%s: Unrecognised option %s:%d:%s\n", conffile, line, tok);
+			ts_error("%s: Unrecognised option %s:%d:%s\n",
+				 conffile, line, tok);
 			break;
 		}
 		if (ret != 0) {
@@ -108,5 +139,36 @@ int ts_config(struct tsdev *ts)
 
 	fclose(f);
 
+	if (strdup_allocated)
+		free(conffile);
+
+	return ret;
+}
+
+int ts_reconfig(struct tsdev *ts)
+{
+	void *handle;
+	int ret;
+	struct tslib_module_info *info, *next;
+	int fd;
+
+	info = ts->list;
+	while (info) {
+		/* Save the "next" pointer now because info will be freed */
+		next = info->next;
+
+		handle = info->handle;
+		info->ops->fini(info);
+		if (handle)
+			dlclose(handle);
+
+		info = next;
+	}
+
+	fd = ts->fd;	/* save temp */
+	memset(ts, 0, sizeof(struct tsdev));
+	ts->fd = fd;
+
+	ret = ts_config(ts);
 	return ret;
 }
